import processing.core.*; 
import processing.xml.*; 

import processing.serial.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class NU32v2_plot extends PApplet {

//////////////////////////////////////////////////////////////////////////////////////
////////////  Nick Marchuk  //////////////////////////////////////////////////////////
////////////  NU32v2 Plotting Application   //////////////////////////////////////////
////////////  February 2011 //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

// serial library
 
Serial myPort; 

// buttons and labels
TextButtons[] serialButtons;
TextButtons printButton;
PFont bigfont, smallfont, divfont;
String serialPortLabel, sentLabel, gotLabel;
int number_serial_buttons;

// boolean controls for program control
boolean SerialOK;

// plot buffers
int[] buffA = new int[1000];
int[] buffB = new int[1000];
int[] buffC = new int[1000];
int[] buffD = new int[1000];
int indA, indB, indC, indD;

//////////////////////////////////////////////////////////////////////////////////////
////////////  setup  /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

public void setup()
{
  size(1030, 602);
  background(230, 230, 230);
  frameRate(30);

  bigfont = loadFont("Gisha-Bold-24.vlw"); 
  smallfont = loadFont("Gisha-Bold-16.vlw");
  divfont = loadFont("Gisha-Bold-8.vlw");

  SerialOK = false;

  serialPortLabel = "No serial port selected, select from 'Serial Ports'";
  sentLabel = "Sent: ";
  gotLabel = "Got: ";

  // serial
  println(Serial.list());
  number_serial_buttons = Serial.list().length;
  serialButtons = new TextButtons[number_serial_buttons];
  int index_serial = 0;
  for (index_serial = 0; index_serial < number_serial_buttons; index_serial++) {
    serialButtons[index_serial] = new TextButtons(50,50+index_serial*40,165,35,bigfont,Serial.list()[index_serial]); // x,y,w,h,font,text
  }
  
  printButton = new TextButtons(850,545,155,35,bigfont,"Print Screen");

  for (int i = 0; i < 1000; i++) {
    buffA[i] = 512;
    buffB[i] = 512;
    buffC[i] = 512;
    buffD[i] = 512;
  }
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  draw  //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

public void draw()
{
  int index = 0, i = 0;
  background(230, 230, 230);

  // status background and text
  fill(255);
  rect(10,532,1000,65);
  fill(0);
  textFont(smallfont);
  text("Status",15,547);

  // status label
  if (SerialOK) {
    fill(15,190,30);
    text(serialPortLabel,15,562); // serial port
    fill(0);
    text(sentLabel,15,580); // what has been sent
    text(gotLabel,15,595); // what the pc has recieved
    text("Plotting",450,562);
    fill(255,0,0);
    text("a",520,562);
    fill(0,255,0);
    text("b",530,562);
    fill(0,0,255);
    text("c",540,562);
    fill(150,15,150);
    text("d",550,562);
  }
  else {
    fill(165,5,5);
    text(serialPortLabel,15,562); // serial port
  }

  // serial buttons
  if (!SerialOK) {
    fill(255);
    rect(45,15,175,35+number_serial_buttons*40); 
    fill(0);
    textFont(smallfont);
    text("Serial Ports",55,40);
    for (index = 0; index < number_serial_buttons; index++) {
      serialButtons[index].update();
      serialButtons[index].display();
    }
    fill(0);
    text("Welcome to the NU32v2 Plotting tool",255,40);

    text("This tool plots four traces, 1000 values long, 10 bit deep.",255,90);
    text("Send information over serial in the form 't index value[newline]'.",255,115);
    text("Where t is a, b, c, or d, index is 0-999, and value is 0-511.",255,140);
    text("a will plot red, b green, c blue, and d purple.",255,165);
    text("Pressing a key will send the pressed character over serial",255,190);
    text("to the microcontroller.",255,215);
    text("The 'Print Screen' button will save a .png image of the screen.",255,240);

    text("Select your COM port on the left to begin.",255,290);
  }
  else {
    printButton.update();
    printButton.display();
    ///////////////////////////////////  Draw the plots /////////////////////////////////////////////////////////////
    textFont(divfont);
    fill(255);
    rect(10,10,1000,512);
    for (i = 10; i < 1000; i=i+100) {
      line(i,10,i,20);
      line(i,512,i,522);
    }
    for (i = 522; i > 10; i=i-100) {
      line(10,i,20,i);
      line(1000,i,1010,i);
    }

    fill(0);
    text("0",1012,525);
    text("100",1012,425);
    text("200",1012,325);
    text("300",1012,225);
    text("400",1012,125);
    text("500",1012,25);

    text("0",10,8);
    text("300",302,8);
    text("500",502,8);
    text("700",702,8);
    text("1000",998,8);

    noStroke();

    for (i = 0; i < 1000; i++) {
      fill(255,0,0);
      ellipse(i+10,522-buffA[i],2,2);

      fill(0,255,0);
      ellipse(i+10,522-buffB[i],2,2);

      fill(0,0,255);
      ellipse(i+10,522-buffC[i],2,2);

      fill(150,15,150);
      ellipse(i+10,522-buffD[i],2,2);
    }
  }
  stroke(0);
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  mouseClicked  //////////////////////////////////////////////////////////
////////////  If a button is selected,  //////////////////////////////////////////////
////////////  perform a function releated to the buttons name  ///////////////////////
//////////////////////////////////////////////////////////////////////////////////////

public void mousePressed() {
  try {
    String buttonText = null;
    int index = 0;
    boolean serialButton = false;

    // check all of the buttons
    for (index = 0; index < number_serial_buttons; index++) {
      if (serialButtons[index].over()) {
        serialButton = true;
        buttonText = serialButtons[index].thetext;
      }
    }
    
    if (printButton.over()) {
      buttonText = printButton.thetext;
    }

    if (buttonText == null) {
      // nothing, no button was clicked
    }
    else {
      /////////////////////////////// Serial Button //////////////////////////////////////////////////////
      if (serialButton) {
        myPort = new Serial(this, buttonText, 115200);
        myPort.bufferUntil('\n'); 
        serialPortLabel = "'" + buttonText + "' selected";
        SerialOK = true;
      }
      ///////////////////////////// Other button /////////////////////////////////////////////////////////
      if (buttonText.equals("Print Screen")) {
        save("printscreen.png");
      }
    }
  }
  catch (NullPointerException e) {
    println("NullPointerException in MousePressed");
    exit();
  }
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  key pressed ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
public void keyReleased() { 

  if (SerialOK) {
    myPort.write(key);
    sentLabel = sentLabel + key;
  }
  else { // not in serial mode yet
    if (key == 'e') {
      exit();
    }
    if (key == '1') {
      myPort = new Serial(this, Serial.list()[0], 115200);
      myPort.bufferUntil('\n'); 
      serialPortLabel = "'" + Serial.list()[0] + "' selected";
      SerialOK = true;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  serial event   /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

public void serialEvent(Serial thisPort) { 
  try {
    if (SerialOK) {
      // read the serial buffer as a string until a newline appears
      String myString = thisPort.readStringUntil('\n');

      // if you got any bytes other than the newline
      if (myString != null) {
/*
        String[] m_a = match(myString, "a");
        String[] m_b = match(myString, "b");
        String[] m_c = match(myString, "c");
        String[] m_d = match(myString, "d");
*/
        if (myString.substring(0, 1).equals("a")) {//(m_a != null) { // data for channel a
          myString = trim(myString); // ditch the newline
          int sensors[] = PApplet.parseInt(split(myString.substring(2), ' '));
          buffA[sensors[0]] = sensors[1];
        }

        else if (myString.substring(0, 1).equals("b")) {//(m_b != null) { // data for channel a
          myString = trim(myString); // ditch the newline
          int sensors[] = PApplet.parseInt(split(myString.substring(2), ' '));
          buffB[sensors[0]] = sensors[1];
        }

        else if (myString.substring(0, 1).equals("c")) {//(m_c != null) { // data for channel a
          myString = trim(myString); // ditch the newline
          int sensors[] = PApplet.parseInt(split(myString.substring(2), ' '));
          buffC[sensors[0]] = sensors[1];
        }

        else if (myString.substring(0, 1).equals("d")) {//(m_d != null) { // data for channel a
          myString = trim(myString); // ditch the newline
          int sensors[] = PApplet.parseInt(split(myString.substring(2), ' '));
          buffD[sensors[0]] = sensors[1];
        }
        
        else {
          myString = trim(myString);
          gotLabel = "Got: " + myString;
        }
      }
    }
  }
  catch (NullPointerException e) {
    println("nullpointer in serial");
  }
} // end serialEvent

//////////////////////////////////////////////////////////////////////////////////////
////////////  exit   /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////


public void stop() {
  //println("inside stop");
  try {
    if (SerialOK) {
      // Clear the buffer, or available() will still be > 0
      myPort.clear();
      // Close the port
      myPort.stop();
    }
    super.stop();
    //println("exiting...");
  }
  catch (NullPointerException e) {
    println("NullPointerException in stop");
  }
} 

//////////////////////////////////////////////////////////////////////////////////////
////////////  button class  //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

class Button
{
  int x, y;
  int w, h;
  int basecolor, highlightcolor, clickcolor;
  PFont thefont;
  boolean over = false;
  boolean pressed = false;   

  public void pressed() {
    if(over && mousePressed) {
      pressed = true;
    } 
    else {
      pressed = false;
    }
  }

  public boolean overRect(int x, int y, int width, int height) {
    if (mouseX >= x && mouseX <= x+width && 
      mouseY >= y && mouseY <= y+height) {
      return true;
    } 
    else {
      return false;
    }
  }
}

class TextButtons extends Button 
{
  String thetext;
  int bgcolor;

  TextButtons(int ix, int iy, int iw, int ih, PFont ithefont, String itext) 
  {
    x = ix;
    y = iy;
    w = iw;
    h = ih;
    thefont = ithefont;
    thetext = itext;
    basecolor = color(52,173,232);
    highlightcolor = color(0,102,153);
    clickcolor = color(152,200,224);//(10,240,100);
  }

  public void update() 
  {
    over();
    pressed();
    if(pressed) {
      bgcolor = clickcolor;
    } 
    else if (over) {
      bgcolor = highlightcolor;
    } 
    else {
      bgcolor = basecolor;
    }
  }

  public boolean over() 
  {
    if( overRect(x, y, w, h) ) {
      over = true;
      return true;
    } 
    else {
      over = false;
      return false;
    }
  }

  public void display() 
  {
    textFont(thefont);
    fill(bgcolor);
    rect(x,y,w,h);
    fill(255,255,255);
    text(thetext,x+5,y+h-10);
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "NU32v2_plot" });
  }
}
